package probichecker;

import java.io.FileReader;
import java.util.HashMap;

import mdp.Bisimulation;
import mdp.MDP;
import mdp.Transition;
import parser.*;
import proc.Proc;

public class CommandLine {

	public static void main( String[] args )  {
		Parser p;
		try {
			p = new Parser( new FileReader(args[0]) );
		} catch (Exception e) {
			throw new Error("can't open file" + e);
		}

		//printTransitions(p);
		//testBisim();
		checkBisimulation(p);
	}
	
	static void checkBisimulation(Parser p) {

		try {
			HashMap<String,Proc> procs = p.Start();
			
			Proc proc1 = procs.get("P1"),
				 proc2 = procs.get("P2");
			if(proc1 == null || proc2 == null)
				throw new Error("you must define process P1 and P2");

			System.out.println("parsed:\n" + proc1 + "\n" + proc2);
			
			System.out.println("translating processes to mdp");
			proc1.translateToStates();
			proc2.translateToStates();
			mdp.MDP mdp = Proc.createMDP();
			
			int state1 = proc1.getStateId();
			int state2 = proc2.getStateId();
			

			//System.out.println("\nstate translation:\n");
			//for(int i = 0; i < Proc.idToProc.size(); i++)
			//	System.out.println(i + ": " + Proc.idToProc.get(i));
			
			//System.out.println("\naction translation:\n");
			//for(int i = 0; i < Proc.idToAction.size(); i++)
			//	System.out.println(i + ": " + Proc.idToAction.get(i));
			
			//System.out.println("mdp:\n" + mdp);
			
			System.out.println("mdp generated, " + mdp.stateNo + " states");
			System.out.println("computing bisimulation classes...");
			Bisimulation bisim = new Bisimulation(mdp);
			bisim.computeClasses();
			System.out.println(
				"\nis bisimilar(" + state1 + ", " + state2 + "): " +
				bisim.isBisimilar(state1, state2)
			);

		} catch(ParseException e) {
			System.out.println("cannot parse input: " + e.getMessage());
		}		
	}
	
	static void testBisim() {
		Transition[][] trans = new Transition[3][];
		trans[0] = new Transition[1];
		trans[0][0] = Transition.dirac(0, 0);
		
		trans[1] = new Transition[1];
		trans[1][0] = new Transition(0, 2);
		trans[1][0].probs[0] = 0.5;
		trans[1][0].states[0] = 0;
		trans[1][0].probs[1] = 0.5;
		trans[1][0].states[1] = 1;		

		trans[2] = new Transition[0];

		MDP mdp = new MDP(3, 2, trans);
		System.out.println("mdp:\n" + mdp);
		Bisimulation bisim = new Bisimulation(mdp);
		bisim.computeClasses();
		System.out.println("bisimulation classes:\n" + bisim.partitions);
	}
	
	static void printTransitions(Parser p) {
		try {
			Proc proc = p.Start().get(0);
			System.out.println("parsed: " + proc);
			System.out.println("transitions:");
			for(proc.Transition t : proc.transitions())
				System.out.println("  " + t);
		} catch(ParseException e) {
			System.out.println("cannot parse input: " + e.getMessage());
		}		
	}
}
